@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2007-2008  Nokia Corporation
  *  Copyright (C) 2007-2008  Instituto Nokia de Tecnologia (INdT)
- *  Copyright (C) 2007-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2007-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -86,6 +86,7 @@ struct obex_commands ftp = {
 
 struct obex_commands pbap = {
 	.get		= pbap_get,
+	.setpath	= pbap_setpath,
 };
 
 static void os_reset_session(struct obex_session *os)
@@ -115,6 +116,7 @@ static void os_reset_session(struct obex_session *os)
 	os->aborted = FALSE;
 	os->offset = 0;
 	os->size = OBJECT_SIZE_DELETE;
+	os->finished = 0;
 }
 
 static void os_session_mark_aborted(struct obex_session *os)
@@ -134,6 +136,9 @@ static void obex_session_free(struct obex_session *os)
 	if (os->current_folder)
 		g_free(os->current_folder);
 
+	if (os->target && !memcmp(os->target, PBAP_TARGET, TARGET_SIZE))
+		pbap_phonebook_context_destroy(os);
+
 	g_free(os);
 }
 
@@ -230,6 +235,7 @@ static void cmd_connect(struct obex_session *os,
 				os->server->services & OBEX_PBAP) {
 			os->target = PBAP_TARGET;
 			os->cmds = &pbap;
+			pbap_phonebook_context_create(os);
 			break;
 		}
 
@@ -466,7 +472,7 @@ static gint obex_write_stream(struct obex_session *os,
 		return -EPERM;
 
 	if (os->fd < 0) {
-		if (os->buf == NULL)
+		if (os->buf == NULL && os->finished == FALSE)
 			return -EIO;
 
 		len = MIN(os->size - os->offset, os->tx_mtu);
@@ -502,6 +508,10 @@ add_header:
 	OBEX_ObjectAddHeader(obex, obj, OBEX_HDR_BODY, hd, len,
 				OBEX_FL_STREAM_DATA);
 
+	if (!memcmp(os->target, PBAP_TARGET, TARGET_SIZE))
+		if (os->offset == os->size && os->finished == FALSE)
+			OBEX_SuspendRequest(obex, obj);
+
 	return len;
 }
 
@@ -512,7 +522,7 @@ gint os_prepare_put(struct obex_session *os)
 
 	path = g_build_filename(os->current_folder, os->name, NULL);
 
-	os->fd = open(path, O_WRONLY | O_CREAT, 0600);
+	os->fd = open(path, O_WRONLY | O_CREAT | O_TRUNC, 0600);
 	if (os->fd < 0) {
 		error("open(%s): %s (%d)", path, strerror(errno), errno);
 		g_free(path);
@@ -727,8 +737,8 @@ static gboolean check_put(obex_t *obex, obex_object_t *obj)
 	}
 
 	free = buf.f_bsize * buf.f_bavail;
-	debug("Free space in disk: %lu", free);
-	if (os->size > free) {
+	debug("Free space in disk: %llu", free);
+	if ((guint64) os->size > free) {
 		debug("Free disk space not available");
 		OBEX_ObjectSetRsp(obj, OBEX_RSP_FORBIDDEN, OBEX_RSP_FORBIDDEN);
 		return FALSE;
@@ -982,6 +992,7 @@ gint obex_session_start(gint fd, struct server *server)
 	}
 
 	OBEX_SetUserData(obex, os);
+	os->obex = obex;
 
 	OBEX_SetTransportMTU(obex, os->rx_mtu, os->tx_mtu);
 
